@@ -1,7 +1,8 @@
-// mars.h - written and placed in the public domain by Wei Dai
+// mars.h - originally written and placed in the public domain by Wei Dai
 
-//! \file mars.h
-//! \brief Classes for the MARS block cipher (IBM AES submission)
+/// \file mars.h
+/// \brief Classes for the MARS block cipher (IBM AES submission)
+/// \since Crypto++ 3.0
 
 #ifndef CRYPTOPP_MARS_H
 #define CRYPTOPP_MARS_H
@@ -11,16 +12,16 @@
 
 NAMESPACE_BEGIN(CryptoPP)
 
-//! \class MARS_Info
-//! \brief MARS block cipher information
+/// \brief MARS block cipher information
+/// \since Crypto++ 3.0
 struct MARS_Info : public FixedBlockSize<16>, public VariableKeyLength<16, 16, 56, 8>
 {
-	CRYPTOPP_CONSTEXPR static const char *StaticAlgorithmName() {return "MARS";}
+	CRYPTOPP_STATIC_CONSTEXPR const char* StaticAlgorithmName() {return "MARS";}
 };
 
-//! \class MARS
-//! \brief MARS block cipher
-//! \sa <a href="http://www.weidai.com/scan-mirror/cs.html#MARS">MARS</a>
+/// \brief MARS block cipher
+/// \sa <a href="http://www.cryptopp.com/wiki/MARS">MARS</a>
+/// \since Crypto++ 3.0
 class MARS : public MARS_Info, public BlockCipherDocumentation
 {
 	class CRYPTOPP_NO_VTABLE Base : public BlockCipherImpl<MARS_Info>
